/* 
 * Copyright (C) 2002-2005 
 * Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, 
 * Boston, MA 02111-1307, USA.
 */

#include "application.h"
#include "lists.h"
#include "check.h"
#include "project.h"
#include "tooltips.h"
#include "search.h"
#include "bookmarks.h"
#include "web_site.h"

#include "callbacks.h"

void
on_view_link_content_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_view_source_dialog_show ();
}

void
on_quit_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_quit ();
}


void
on_about_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_about_dialog_show ();
}


void
on_settings_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_settings_dialog_show ();
}


void
on_display_badlinks_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_status (UC_CHECK_DISPLAY_BADLINKS);
}


void
on_display_goodlinks_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_status (UC_CHECK_DISPLAY_GOODLINKS);
}

void
on_display_documents_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_type (LINK_TYPE_HREF);
}

void
on_display_images_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_type (LINK_TYPE_IMAGE);
}

void
on_display_stylesheets_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_type (LINK_TYPE_CSS);
}

void
on_display_frames_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_type (LINK_TYPE_FRAME);
}

void
on_display_emails_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_type (LINK_TYPE_EMAIL);
}

void
on_display_malformedlinks_activate (GtkMenuItem * menuitem,
				    gpointer user_data)
{
  uc_application_main_tree_display_status (UC_CHECK_DISPLAY_MALFORMEDLINKS);
}

void
on_display_timeouts_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_status (UC_CHECK_DISPLAY_TIMEOUTS);
}

void
on_display_http_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_protocol (UC_PROTOCOL_HTTP);
}

void
on_display_https_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_protocol (UC_PROTOCOL_HTTPS);
}

void
on_display_ftp_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_protocol (UC_PROTOCOL_FTP);
}

void
on_display_all_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_display_all ();
}


void
on_display_collapse_all_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_collapse_all ();
}


void
on_display_expand_all_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_main_tree_expand_all ();
}

void
on_bt_cancel_settings_dialog_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (settings_dialog), settings_dialog = NULL;
}


void
on_bt_ok_settings_dialog_clicked (GtkButton * button, gpointer user_data)
{
  if (uc_application_settings_get_data ())
    gtk_widget_destroy (settings_dialog), settings_dialog = NULL;
}

void
on_bt_new_project_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_new_web_dialog_show ();
}

void
on_bt_apply_settings_dialog_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_settings_get_data ();
}


void
on_treeview_projects_cursor_changed (GtkTreeView * selection,
				     gpointer user_data)
{
  uc_project_display_informations (selection);
}


void
on_urls_list_cursor_changed (GtkTreeView * selection, gpointer user_data)
{
  uc_application_display_informations (selection);
}

void
on_search_list_cursor_changed (GtkTreeView * selection, gpointer user_data)
{
  UC_UPDATE_UI;
  uc_search_main_select (selection);
}

gboolean
on_url_list_mouse_clicked (GtkWidget * widget, GdkEventButton * event,
			   gpointer data)
{
  /* show page properties on left
   * double-click */
  if (event->type == GDK_2BUTTON_PRESS)
    uc_application_page_information_dialog_show ();

  /* show contextual popup menu on right
   * button release */
  else if ((event->type == GDK_BUTTON_RELEASE) && (event->button == 3))
    uc_application_treeview_activate_popup (event);

  return FALSE;
}

gboolean
on_project_list_mouse_clicked (GtkWidget * widget, GdkEventButton * event,
			       gpointer data)
{
  /* open project on left double-click */
  if (event->type == GDK_2BUTTON_PRESS)
    {
      guint32 id = 0;

      id = uc_project_treeview_get_selected_row_id ();

      gtk_widget_destroy (open_project_dialog), open_project_dialog = NULL;

      uc_project_open (id);
    }

  return FALSE;
}

void
on_bt_cancel_progress_dialog_clicked (GtkButton * button, gpointer user_data)
{
  uc_check_cancel_set_value (TRUE);
}

void
on_bt_ignore_progress_dialog_clicked (GtkButton * button, gpointer user_data)
{
  uc_check_ignore_item_set_value (TRUE);
}

void
on_progress_dialog_close (GtkDialog * dialog, gpointer user_data)
{
  uc_check_cancel_set_value (TRUE);
}

gboolean
on_progress_dialog_destroy_event (GtkWidget * widget,
				  GdkEvent * event, gpointer user_data)
{
  uc_check_cancel_set_value (TRUE);
  return TRUE;
}


gboolean
on_progress_dialog_delete_event (GtkWidget * widget,
				 GdkEvent * event, gpointer user_data)
{
  uc_check_cancel_set_value (TRUE);
  return TRUE;
}


void
on_progress_dialog_destroy (GtkObject * object, gpointer user_data)
{
  uc_check_cancel_set_value (TRUE);
}


void
on_bt_basic_display_format_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_display_informations (GTK_TREE_VIEW (user_data));
}

void
on_bt_detailed_display_format_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_display_informations (GTK_TREE_VIEW (user_data));
}

void
on_bt_close_view_source_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (view_source_dialog), view_source_dialog = NULL;
}

gboolean
on_entry_project_url_key_press_event (GtkWidget * widget,
				      GdkEventKey * event, gpointer user_data)
{
  if (event->keyval != GDK_Return)
    return FALSE;

  gtk_widget_activate (glade_xml_get_widget
		       (glade_new_project_dialog, "bt_ok"));

  return TRUE;
}

void
on_new_instance_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_new_instance_launch ();
}

void
on_bt_suspend_progress_dialog_clicked (GtkButton * button, gpointer user_data)
{
  uc_check_suspend_continue ();
}


void
on_bt_close_similar_links_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (similar_links_dialog), similar_links_dialog = NULL;
}


void on_view_similar_link_locations_activate
  (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_view_similar_links_dialog_show ();

}

void
on_view_online_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  UCLinkProperties *prop = NULL;
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    {
      prop = uc_lists_checked_links_lookup_by_uid (id);
      uc_application_launch_web_browser (prop->url);
    }
  else
    {
      uc_application_dialog_show (_
				  ("Please, select a item first."),
				  GTK_MESSAGE_WARNING);
    }
}

void
on_page_properties_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    uc_application_page_information_dialog_show ();
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_WARNING);
}


void
on_view_parent_page_online_activate (GtkMenuItem * menuitem,
				     gpointer user_data)
{
  UCLinkProperties *prop = NULL;
  gchar *parent_url = NULL;
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    {
      prop = uc_lists_checked_links_lookup_by_uid (id);
      parent_url =
	(prop->parent) ? (prop->parent)->url : uc_project_get_url ();
      uc_application_launch_web_browser (parent_url);
    }
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_WARNING);
}

void
on_bt_close_page_information_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (page_information_dialog), page_information_dialog =
    NULL;
}


void on_bt_cancel_add_filter_directory_dialog_clicked
  (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (add_filter_directory_dialog),
    add_filter_directory_dialog = NULL;
}

void on_bt_cancel_add_filter_domain_dialog_clicked
  (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (add_filter_domain_dialog),
    add_filter_domain_dialog = NULL;
}

void
on_bt_add_filter_directory_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_add_filter_directory_dialog_show ();
}


void
on_bt_add_filter_domain_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_add_filter_domain_dialog_show ();
}


gboolean on_entry_add_filter_directory_key_press_event
  (GtkWidget * widget, GdkEventKey * event, gpointer user_data)
{
  if (event->keyval != GDK_Return)
    return FALSE;

  gtk_widget_activate (glade_xml_get_widget
		       (glade_add_filter_directory_dialog,
			"bt_add_add_filter_directory_dialog"));

  return TRUE;
}


void on_bt_add_add_filter_directory_dialog_clicked
  (GtkButton * button, gpointer user_data)
{
  uc_application_add_filter_directory_add ();
}


void on_bt_add_add_filter_domain_dialog_clicked
  (GtkButton * button, gpointer user_data)
{
  uc_application_add_filter_domain_add ();
}


gboolean on_entry_add_filter_domain_key_press_event
  (GtkWidget * widget, GdkEventKey * event, gpointer user_data)
{
  if (event->keyval != GDK_Return)
    return FALSE;

  gtk_widget_activate (glade_xml_get_widget
		       (glade_add_filter_domain_dialog,
			"bt_add_add_filter_domain_dialog"));

  return TRUE;
}


void
on_bt_remove_filter_domain_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_add_filter_domain_remove ();
}


void
on_bt_remove_filter_directory_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_add_filter_directory_remove ();
}


void
on_bt_clear_filter_directory_clicked (GtkButton * button, gpointer user_data)
{
  gtk_list_store_clear (treestore_filter_directories);
}


void
on_bt_clear_filter_domain_clicked (GtkButton * button, gpointer user_data)
{
  gtk_list_store_clear (treestore_filter_domains);
}

void
on_check_email_address_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *widget1 = NULL;
  GtkWidget *widget2 = NULL;
  gboolean active = FALSE;

  widget1 =
    glade_xml_get_widget (glade_settings_dialog, "check_email_address");
  widget2 = glade_xml_get_widget (glade_settings_dialog, "check_email_mx");

  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget1));
  if (active == FALSE)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget2), FALSE);

  gtk_widget_set_sensitive (GTK_WIDGET (widget2), active);
}


void
on_menu_check_email_mx_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_check_mx_is_valid ();
}

void
on_image_preview_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_view_image_dialog_show ();
}


void
on_bt_close_image_preview_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (view_image_dialog), view_image_dialog = NULL;
}

void
  on_dynamic_values_time_between_value_changed
  (GtkSpinButton * spinbutton, gpointer user_data)
{
  GtkWidget *widget = NULL;

  widget =
    glade_xml_get_widget (glade_progress_dialog,
			  "dynamic_values_time_between");

  uc_project_set_check_wait ((guint)
			     gtk_spin_button_get_value (GTK_SPIN_BUTTON
							(widget)));
}


void
  on_dynamic_values_timeout_value_changed
  (GtkSpinButton * spinbutton, gpointer user_data)
{
  GtkWidget *widget = NULL;

  widget =
    glade_xml_get_widget (glade_progress_dialog, "dynamic_values_timeout");

  uc_project_set_check_timeout ((guint)
				gtk_spin_button_get_value (GTK_SPIN_BUTTON
							   (widget)));
}

void
on_search_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_new_search_dialog_show ();
}


void
on_new_search_button_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_search_get_data ();
}


void
on_new_search_cancel_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (new_search_dialog), new_search_dialog = NULL;
}


void
on_search_close_clicked (GtkButton * button, gpointer user_data)
{
  uc_search_set_exit (TRUE);

  if (treestore_search != NULL)
    g_object_unref (G_OBJECT (treestore_search)), treestore_search = NULL;

  WSENS (glade_main_window, "find", TRUE);
  WSENS (glade_main_window, "project", TRUE);
  WSENS (glade_main_window, "bt_new", TRUE);
  WSENS (glade_main_window, "bt_open", TRUE);
}


void
on_clear_project_history_clicked (GtkButton * button, gpointer user_data)
{
  gnome_entry_clear_history (GNOME_ENTRY (gnome_entry_new ("web_project")));
  gnome_entry_clear_history (GNOME_ENTRY
			     (gnome_entry_new ("bookmarks_project")));
  gnome_entry_clear_history (GNOME_ENTRY
			     (gnome_entry_new ("local_file_project")));
}


void
on_clear_search_history_clicked (GtkButton * button, gpointer user_data)
{
  gnome_entry_clear_history (GNOME_ENTRY (gnome_entry_new ("search")));
}


void
on_new_search_check_button_toggled (GtkToggleButton * togglebutton,
				    gpointer user_data)
{
  uc_search_check_verification ();
}

gboolean
on_search_dialog_delete_event (GtkWidget * widget,
			       GdkEvent * event, gpointer user_data)
{
  on_search_close_clicked (NULL, NULL);

  return FALSE;
}


gboolean
on_combo_search_key_press_event (GtkWidget * widget,
				 GdkEventKey * event, gpointer user_data)
{
  if (event->keyval != GDK_Return)
    return FALSE;

  gtk_widget_activate (glade_xml_get_widget
		       (glade_new_search_dialog, "bt_search"));

  return TRUE;
}

void
on_web_site_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_new_web_dialog_show ();
}


void
on_local_file_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_new_local_file_dialog_show ();
}


void
on_bookmarks_file_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_new_bookmarks_dialog_show ();
}

void
on_bt_cancel_new_bookmarks_project_clicked (GtkButton * button,
					    gpointer user_data)
{
  gtk_widget_destroy (new_project_dialog), new_project_dialog = NULL;
}


void
on_bt_ok_new_bookmarks_project_clicked (GtkButton * button,
					gpointer user_data)
{
  uc_application_get_bookmarks_project_data ();
}

gboolean
on_bookmarks_key_press_event (GtkWidget * widget,
			      GdkEventKey * event, gpointer user_data)
{
  if (event->keyval != GDK_Return)
    return FALSE;

  gtk_widget_activate (glade_xml_get_widget
		       (glade_new_project_dialog, "bt_ok"));

  return TRUE;
}

void
on_delete_link_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    uc_bookmarks_delete_link (id);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_WARNING);
}

void
on_save_bookmarks_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_save_bookmarks ();
}

void
on_save_project_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_project_save ();
}


void
on_open_project_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  uc_application_open_project_dialog_show ();
}

void
on_project_properties_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  (gboolean) uc_application_project_information_dialog_show ();
}

void
on_menu_refresh_all_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  if (uc_project_get_bookmarks_file ())
    g_warning ("function not yet implemented!");	/* FIXME */
  else
    uc_web_site_refresh_all ();
}

void
on_menu_refresh_branch_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    uc_web_site_refresh_branch (id);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_WARNING);
}

void
on_menu_refresh_link_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    uc_web_site_refresh_link (id);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_WARNING);
}

void
on_bt_close_open_project_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (open_project_dialog), open_project_dialog = NULL;
  uc_application_set_status_bar (0, "");
}


void
on_bt_ok_open_project_clicked (GtkButton * button, gpointer user_data)
{
  gint32 id = 0;

  id = uc_project_treeview_get_selected_row_id ();
  if (id >= 0)
    {
      gtk_widget_destroy (open_project_dialog), open_project_dialog = NULL;
      uc_project_open (id);
    }
  else
    uc_application_dialog_show (_("Please, select a project to open."),
				GTK_MESSAGE_WARNING);
}


void
on_bt_delete_open_project_clicked (GtkButton * button, gpointer user_data)
{
  gint32 id = 0;

  id = uc_project_treeview_get_selected_row_id ();
  if (id >= 0)
    {
      if (uc_application_dialog_yes_no_show
	  (_("Are you sure you want to delete " "this project?"),
	   GTK_MESSAGE_QUESTION))
	uc_project_delete (id);
    }
  else
    uc_application_dialog_show (_("Please, select a project to delete."),
				GTK_MESSAGE_WARNING);
}

void
on_bt_cancel_new_web_project_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (new_project_dialog), new_project_dialog = NULL;
}

void
on_bt_ok_new_web_project_clicked (GtkButton * button, gpointer user_data)
{
  uc_application_project_get_data (UC_PROJECT_TYPE_WEB_SITE);
}

void
on_bt_ok_new_local_file_project_clicked (GtkButton * button,
					 gpointer user_data)
{
  uc_application_project_get_data (UC_PROJECT_TYPE_LOCAL_FILE);
}

gboolean
on_main_window_delete_event (GtkWidget * widget,
			     GdkEvent * event, gpointer user_data)
{
  uc_application_quit ();

  return FALSE;
}

gboolean
on_main_treeview_motion_notify_event (GtkWidget * widget,
				      GdkEventButton * event, gpointer data)
{
  uc_tooltips_main_set_mouse_coord (event->x, event->y);

  return FALSE;
}

/**
 * on_main_treeview_enter_notify_event:
 * @widget: Widget.
 * @event: Event.
 * @user_data: Nothing.
 *
 * When mouse enter in the tree view zone. Mainly used by main tree view
 * tooltips.
 * 
 * Returns: Always FALSE.
 */
gboolean
on_main_treeview_enter_notify_event (GtkWidget * widget,
				     GdkEventCrossing * event,
				     gpointer user_data)
{
  uc_tooltips_main_set_frozen (FALSE);

  return FALSE;
}

/**
 * on_main_treeview_leave_notify_event:
 * @widget: Widget.
 * @event: Event.
 * @user_data: Noting.
 *
 * When mouse leace the tree view zone. Mainly used by main tree view 
 * tooltips.
 *
 * Returns: Always FALSE.
 */
gboolean
on_main_treeview_leave_notify_event (GtkWidget * widget,
				     GdkEventCrossing * event,
				     gpointer user_data)
{
  uc_tooltips_main_set_frozen (TRUE);

  if (uc_tooltips_main_get_active ())
    {
      uc_tooltips_main_set_active (FALSE);
      uc_tooltips_main_destroy ();
    }

  return FALSE;
}

void
on_display_tooltips_toggled (GtkToggleButton * togglebutton,
			     gpointer user_data)
{
  uc_tooltips_main_set_display (gtk_toggle_button_get_active (togglebutton));
}

void
on_delete_project_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  if (uc_application_dialog_yes_no_show
      (_("Are you sure you want to delete " "this project?"),
       GTK_MESSAGE_QUESTION))
    uc_project_delete (uc_project_get_id ());
}

void
on_menu_refresh_parent_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  gint32 id = 0;

  id = uc_check_treeview_get_selected_row_id ();
  if (id >= 0)
    uc_web_site_refresh_parent (id);
  else
    uc_application_dialog_show (_
				("Please, select a item first."),
				GTK_MESSAGE_WARNING);
}

void
on_menu_refresh_main_page_activate (GtkMenuItem * menuitem,
				    gpointer user_data)
{
  uc_web_site_refresh_link (uc_check_get_main_page_id ());
}

gboolean
on_open_project_dialog_delete_event (GtkWidget * widget,
				     GdkEvent * event, gpointer user_data)
{
  uc_application_set_status_bar (0, "");

  return FALSE;
}
